@@ -5,6 +5,6 @@ mod backend;
 pub mod error;
 pub mod tool;
 
-pub use backend::{Backend, BackendOutputStream, json_rpc};
+pub use backend::{Backend, BackendOutputStream, in_memory, json_rpc};
 pub use backend_override::BackendOverride;
 pub use pixi_build_types as types;
