@@ -3,10 +3,11 @@ use pixi_build_discovery::{
     BackendInitializationParams, BackendSpec, CommandSpec, EnabledProtocols,
 };
 use pixi_build_frontend::{
-    Backend, json_rpc,
-    json_rpc::JsonRpcBackend,
+    Backend, BackendOverride, json_rpc,
+    json_rpc::{CommunicationError, JsonRpcBackend},
     tool::{IsolatedTool, SystemTool, Tool},
 };
+use pixi_build_types::{PixiBuildApiVersion, procedures::initialize::InitializeParams};
 use pixi_spec::PixiSpec;
 use rattler_conda_types::ChannelConfig;
 use rattler_shell::{
@@ -19,7 +20,10 @@ use thiserror::Error;
 use crate::{
     BuildEnvironment, CommandDispatcher, CommandDispatcherErrorResultExt,
     command_dispatcher::error::CommandDispatcherError,
-    instantiate_tool_env::{InstantiateToolEnvironmentError, InstantiateToolEnvironmentSpec},
+    instantiate_tool_env::{
+        InstantiateToolEnvironmentError, InstantiateToolEnvironmentResult,
+        InstantiateToolEnvironmentSpec,
+    },
 };
 
 #[derive(Debug)]
@@ -46,38 +50,45 @@ impl CommandDispatcher {
     ) -> Result<Backend, CommandDispatcherError<InstantiateBackendError>> {
         let BackendSpec::JsonRpc(backend_spec) = spec.backend_spec;
 
-        let command_spec = self
-            .build_backend_overrides()
-            .named_backend_override(&backend_spec.name)
-            .unwrap_or(backend_spec.command);
+        let command_spec = match self.build_backend_overrides() {
+            BackendOverride::System(overridden_backends) => overridden_backends
+                .named_backend_override(&backend_spec.name)
+                .unwrap_or(backend_spec.command),
+            BackendOverride::InMemory(memory) => {
+                let backend = memory
+                    .initialize(InitializeParams {
+                        manifest_path: spec.init_params.manifest_path,
+                        source_dir: Some(spec.init_params.source_dir),
+                        cache_directory: Some(self.cache_dirs().root().clone()),
+                        project_model: spec.init_params.project_model.map(Into::into),
+                        configuration: spec.init_params.configuration,
+                    })
+                    .map_err(InstantiateBackendError::InMemoryError)
+                    .map_err(CommandDispatcherError::Failed)?;
+                return Ok(Backend::new(backend.into(), memory.api_version()));
+            }
+        };
 
-        let tool = match command_spec {
-            CommandSpec::System(system_spec) => Tool::System(SystemTool::new(
-                system_spec.command.unwrap_or(backend_spec.name),
-            )),
+        let (tool, api_version) = match command_spec {
+            CommandSpec::System(system_spec) => (
+                Tool::System(SystemTool::new(
+                    system_spec.command.unwrap_or(backend_spec.name),
+                )),
+                // Assume the latest version of the backend
+                PixiBuildApiVersion::current(),
+            ),
             CommandSpec::EnvironmentSpec(env_spec) => {
                 let (tool_platform, tool_platform_virtual_packages) = self.tool_platform();
-                let prefix = self
+                let InstantiateToolEnvironmentResult { prefix, api } = self
                     .instantiate_tool_environment(InstantiateToolEnvironmentSpec {
                         requirement: (
                             env_spec.requirement.0,
-                            PixiSpec::from_nameless_matchspec(
-                                env_spec.requirement.1,
-                                &spec.channel_config,
-                            ),
+                            PixiSpec::from(env_spec.requirement.1),
                         ),
                         additional_requirements: env_spec
                             .additional_requirements
                             .into_specs()
-                            .map(|(name, nameless)| {
-                                (
-                                    name,
-                                    PixiSpec::from_nameless_matchspec(
-                                        nameless,
-                                        &spec.channel_config,
-                                    ),
-                                )
-                            })
+                            .map(|(name, spec)| (name, PixiSpec::from(spec)))
                             .collect(),
                         constraints: env_spec.constraints,
                         build_environment: BuildEnvironment {
@@ -106,11 +117,14 @@ impl CommandDispatcher {
                     .map_err(InstantiateBackendError::from)
                     .map_err(CommandDispatcherError::Failed)?;
 
-                Tool::from(IsolatedTool::new(
-                    env_spec.command.unwrap_or(backend_spec.name),
-                    prefix.path().to_path_buf(),
-                    activation_scripts,
-                ))
+                (
+                    Tool::from(IsolatedTool::new(
+                        env_spec.command.unwrap_or(backend_spec.name),
+                        prefix.path().to_path_buf(),
+                        activation_scripts,
+                    )),
+                    api,
+                )
             }
         };
 
@@ -133,7 +147,7 @@ impl CommandDispatcher {
         .await
         .map_err(InstantiateBackendError::from)
         .map_err(CommandDispatcherError::Failed)
-        .map(Backend::JsonRpc)
+        .map(|backend| Backend::new(backend.into(), api_version))
     }
 }
 
@@ -144,6 +158,11 @@ pub enum InstantiateBackendError {
     #[diagnostic(transparent)]
     JsonRpc(#[from] json_rpc::InitializeError),
 
+    /// The command dispatcher could not be initialized.
+    #[error(transparent)]
+    #[diagnostic(transparent)]
+    InMemoryError(CommunicationError),
+
     /// Could not detect the virtual packages for the system
     #[error(transparent)]
     VirtualPackages(#[from] DetectVirtualPackageError),
