@@ -2,13 +2,15 @@ use std::sync::{Arc, Mutex};
 
 use futures::{Stream, StreamExt};
 use pixi_command_dispatcher::{
-    CondaSolveReporter, GitCheckoutReporter, InstallPixiEnvironmentSpec,
-    InstantiateToolEnvironmentSpec, PixiEnvironmentSpec, PixiInstallReporter, PixiSolveReporter,
-    Reporter, ReporterContext, SolveCondaEnvironmentSpec, SourceBuildSpec, SourceMetadataSpec,
+    BackendSourceBuildSpec, BuildBackendMetadataSpec, CondaSolveReporter, GitCheckoutReporter,
+    InstallPixiEnvironmentSpec, InstantiateToolEnvironmentSpec, PackageIdentifier,
+    PixiEnvironmentSpec, PixiInstallReporter, PixiSolveReporter, Reporter, ReporterContext,
+    SolveCondaEnvironmentSpec, SourceBuildSpec, SourceMetadataSpec,
     reporter::{
-        CondaSolveId, GitCheckoutId, InstantiateToolEnvId, InstantiateToolEnvironmentReporter,
-        PixiInstallId, PixiSolveId, SourceBuildId, SourceBuildReporter, SourceMetadataId,
-        SourceMetadataReporter,
+        BackendSourceBuildId, BackendSourceBuildReporter, BuildBackendMetadataId,
+        BuildBackendMetadataReporter, CondaSolveId, GitCheckoutId, InstantiateToolEnvId,
+        InstantiateToolEnvironmentReporter, PixiInstallId, PixiSolveId, SourceBuildId,
+        SourceBuildReporter, SourceMetadataId, SourceMetadataReporter,
     },
 };
 use pixi_git::resolver::RepositoryReference;
@@ -73,6 +75,20 @@ pub enum Event {
         id: GitCheckoutId,
     },
 
+    BuildBackendMetadataQueued {
+        id: BuildBackendMetadataId,
+        #[serde(flatten)]
+        spec: BuildBackendMetadataSpec,
+        #[serde(skip_serializing_if = "Option::is_none")]
+        context: Option<ReporterContext>,
+    },
+    BuildBackendMetadataStarted {
+        id: BuildBackendMetadataId,
+    },
+    BuildBackendMetadataFinished {
+        id: BuildBackendMetadataId,
+    },
+
     SourceMetadataQueued {
         id: SourceMetadataId,
         #[serde(flatten)]
@@ -101,6 +117,19 @@ pub enum Event {
         id: SourceBuildId,
     },
 
+    BackendSourceBuildQueued {
+        id: BackendSourceBuildId,
+        #[serde(skip_serializing_if = "Option::is_none")]
+        context: Option<ReporterContext>,
+        package: PackageIdentifier,
+    },
+    BackendSourceBuildStarted {
+        id: BackendSourceBuildId,
+    },
+    BackendSourceBuildFinished {
+        id: BackendSourceBuildId,
+    },
+
     InstantiateToolEnvQueued {
         id: InstantiateToolEnvId,
         #[serde(flatten)]
@@ -158,20 +187,20 @@ impl CondaSolveReporter for EventReporter {
             spec: Box::new(env.clone()),
             context,
         };
-        println!("{}", serde_json::to_string_pretty(&event).unwrap());
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
         self.events.lock().unwrap().push(event);
         next_id
     }
 
     fn on_start(&mut self, solve_id: CondaSolveId) {
         let event = Event::CondaSolveStarted { id: solve_id };
-        println!("{}", serde_json::to_string_pretty(&event).unwrap());
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
         self.events.lock().unwrap().push(event);
     }
 
     fn on_finished(&mut self, solve_id: CondaSolveId) {
         let event = Event::CondaSolveFinished { id: solve_id };
-        println!("{}", serde_json::to_string_pretty(&event).unwrap());
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
         self.events.lock().unwrap().push(event);
     }
 }
@@ -190,20 +219,20 @@ impl PixiSolveReporter for EventReporter {
             spec: env.clone(),
             context,
         };
-        println!("{}", serde_json::to_string_pretty(&event).unwrap());
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
         self.events.lock().unwrap().push(event);
         next_id
     }
 
     fn on_start(&mut self, solve_id: PixiSolveId) {
         let event = Event::PixiSolveStarted { id: solve_id };
-        println!("{}", serde_json::to_string_pretty(&event).unwrap());
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
         self.events.lock().unwrap().push(event);
     }
 
     fn on_finished(&mut self, solve_id: PixiSolveId) {
         let event = Event::PixiSolveFinished { id: solve_id };
-        println!("{}", serde_json::to_string_pretty(&event).unwrap());
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
         self.events.lock().unwrap().push(event);
     }
 }
@@ -222,20 +251,20 @@ impl PixiInstallReporter for EventReporter {
             spec: env.clone(),
             context,
         };
-        println!("{}", serde_json::to_string_pretty(&event).unwrap());
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
         self.events.lock().unwrap().push(event);
         next_id
     }
 
     fn on_start(&mut self, solve_id: PixiInstallId) {
         let event = Event::PixiInstallStarted { id: solve_id };
-        println!("{}", serde_json::to_string_pretty(&event).unwrap());
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
         self.events.lock().unwrap().push(event);
     }
 
     fn on_finished(&mut self, solve_id: PixiInstallId) {
         let event = Event::PixiInstallFinished { id: solve_id };
-        println!("{}", serde_json::to_string_pretty(&event).unwrap());
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
         self.events.lock().unwrap().push(event);
     }
 }
@@ -254,20 +283,52 @@ impl GitCheckoutReporter for EventReporter {
             reference: env.clone(),
             context,
         };
-        println!("{}", serde_json::to_string_pretty(&event).unwrap());
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
         self.events.lock().unwrap().push(event);
         next_id
     }
 
     fn on_start(&mut self, checkout_id: GitCheckoutId) {
         let event = Event::GitCheckoutStarted { id: checkout_id };
-        println!("{}", serde_json::to_string_pretty(&event).unwrap());
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
         self.events.lock().unwrap().push(event);
     }
 
     fn on_finished(&mut self, checkout_id: GitCheckoutId) {
         let event = Event::GitCheckoutFinished { id: checkout_id };
-        println!("{}", serde_json::to_string_pretty(&event).unwrap());
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
+        self.events.lock().unwrap().push(event);
+    }
+}
+
+impl BuildBackendMetadataReporter for EventReporter {
+    fn on_queued(
+        &mut self,
+        context: Option<ReporterContext>,
+        spec: &BuildBackendMetadataSpec,
+    ) -> BuildBackendMetadataId {
+        let next_id = BuildBackendMetadataId(self.next_source_metadata_id);
+        self.next_source_metadata_id += 1;
+
+        let event = Event::BuildBackendMetadataQueued {
+            id: next_id,
+            spec: spec.clone(),
+            context,
+        };
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
+        self.events.lock().unwrap().push(event);
+        next_id
+    }
+
+    fn on_started(&mut self, id: BuildBackendMetadataId) {
+        let event = Event::BuildBackendMetadataStarted { id };
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
+        self.events.lock().unwrap().push(event);
+    }
+
+    fn on_finished(&mut self, id: BuildBackendMetadataId) {
+        let event = Event::BuildBackendMetadataFinished { id };
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
         self.events.lock().unwrap().push(event);
     }
 }
@@ -286,20 +347,20 @@ impl SourceMetadataReporter for EventReporter {
             spec: spec.clone(),
             context,
         };
-        println!("{}", serde_json::to_string_pretty(&event).unwrap());
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
         self.events.lock().unwrap().push(event);
         next_id
     }
 
     fn on_started(&mut self, id: SourceMetadataId) {
         let event = Event::SourceMetadataStarted { id };
-        println!("{}", serde_json::to_string_pretty(&event).unwrap());
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
         self.events.lock().unwrap().push(event);
     }
 
     fn on_finished(&mut self, id: SourceMetadataId) {
         let event = Event::SourceMetadataFinished { id };
-        println!("{}", serde_json::to_string_pretty(&event).unwrap());
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
         self.events.lock().unwrap().push(event);
     }
 }
@@ -318,20 +379,20 @@ impl InstantiateToolEnvironmentReporter for EventReporter {
             spec: spec.clone(),
             context,
         };
-        println!("{}", serde_json::to_string_pretty(&event).unwrap());
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
         self.events.lock().unwrap().push(event);
         next_id
     }
 
     fn on_started(&mut self, id: InstantiateToolEnvId) {
         let event = Event::InstantiateToolEnvStarted { id };
-        println!("{}", serde_json::to_string_pretty(&event).unwrap());
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
         self.events.lock().unwrap().push(event);
     }
 
     fn on_finished(&mut self, id: InstantiateToolEnvId) {
         let event = Event::InstantiateToolEnvFinished { id };
-        println!("{}", serde_json::to_string_pretty(&event).unwrap());
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
         self.events.lock().unwrap().push(event);
     }
 }
@@ -350,31 +411,63 @@ impl SourceBuildReporter for EventReporter {
             spec: spec.clone(),
             context,
         };
-        println!("{}", serde_json::to_string_pretty(&event).unwrap());
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
+        self.events.lock().unwrap().push(event);
+        next_id
+    }
+
+    fn on_started(&mut self, id: SourceBuildId) {
+        let event = Event::SourceBuildStarted { id };
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
+        self.events.lock().unwrap().push(event);
+    }
+
+    fn on_finished(&mut self, id: SourceBuildId) {
+        let event = Event::SourceBuildFinished { id };
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
+        self.events.lock().unwrap().push(event);
+    }
+}
+
+impl BackendSourceBuildReporter for EventReporter {
+    fn on_queued(
+        &mut self,
+        context: Option<ReporterContext>,
+        spec: &BackendSourceBuildSpec,
+    ) -> BackendSourceBuildId {
+        let next_id = BackendSourceBuildId(self.next_source_metadata_id);
+        self.next_source_metadata_id += 1;
+
+        let event = Event::BackendSourceBuildQueued {
+            id: next_id,
+            context,
+            package: spec.package.clone(),
+        };
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
         self.events.lock().unwrap().push(event);
         next_id
     }
 
     fn on_started(
         &mut self,
-        id: SourceBuildId,
+        id: BackendSourceBuildId,
         backend_output_stream: Box<dyn Stream<Item = String> + Unpin + Send>,
     ) {
-        let event = Event::SourceBuildStarted { id };
-        println!("{}", serde_json::to_string_pretty(&event).unwrap());
+        let event = Event::BackendSourceBuildStarted { id };
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
         self.events.lock().unwrap().push(event);
 
         tokio::spawn(async move {
             let mut output_stream = backend_output_stream;
             while let Some(line) = output_stream.next().await {
-                println!("{}", line);
+                eprintln!("{}", line);
             }
         });
     }
 
-    fn on_finished(&mut self, id: SourceBuildId) {
-        let event = Event::SourceBuildFinished { id };
-        println!("{}", serde_json::to_string_pretty(&event).unwrap());
+    fn on_finished(&mut self, id: BackendSourceBuildId) {
+        let event = Event::BackendSourceBuildFinished { id };
+        eprintln!("{}", serde_json::to_string_pretty(&event).unwrap());
         self.events.lock().unwrap().push(event);
     }
 }
@@ -402,10 +495,19 @@ impl Reporter for EventReporter {
         Some(self)
     }
 
+    fn as_build_backend_metadata_reporter(
+        &mut self,
+    ) -> Option<&mut dyn BuildBackendMetadataReporter> {
+        Some(self)
+    }
     fn as_source_metadata_reporter(&mut self) -> Option<&mut dyn SourceMetadataReporter> {
         Some(self)
     }
     fn as_source_build_reporter(&mut self) -> Option<&mut dyn SourceBuildReporter> {
         Some(self)
     }
+
+    fn as_backend_source_build_reporter(&mut self) -> Option<&mut dyn BackendSourceBuildReporter> {
+        Some(self)
+    }
 }
