@@ -1,9 +1,10 @@
-use std::{collections::HashSet, hash::Hash, path::PathBuf};
+use std::{hash::Hash, path::PathBuf};
 
+use indexmap::IndexMap;
 use indexmap::IndexSet;
 use pep508_rs::PackageName;
-use serde::ser::SerializeSeq;
-use serde::{Serialize, Serializer};
+use pixi_pypi_spec::{PixiPypiSpec, PypiPackageName};
+use serde::{Serialize, Serializer, ser::SerializeSeq};
 use thiserror::Error;
 use url::Url;
 
@@ -62,7 +63,7 @@ pub enum NoBuild {
     All,
     /// Don't build sdist for specific packages
     // Todo: would be nice to check if these are actually used at some point
-    Packages(HashSet<pep508_rs::PackageName>),
+    Packages(IndexSet<pep508_rs::PackageName>),
 }
 
 impl NoBuild {
@@ -84,6 +85,39 @@ impl NoBuild {
     }
 }
 
+/// Don't install pre-built wheels for all or certain packages
+#[derive(Default, Debug, Clone, PartialEq, Eq, Serialize, strum::Display)]
+#[strum(serialize_all = "kebab-case")]
+#[serde(rename_all = "kebab-case")]
+pub enum NoBinary {
+    /// Use pre-built wheels for all package
+    #[default]
+    None,
+    /// Build all package from source
+    All,
+    /// Build specific packages from source
+    Packages(IndexSet<pep508_rs::PackageName>),
+}
+
+impl NoBinary {
+    /// Merges two `NoBinary` together, according to the following rules
+    /// - If either is `All`, the result is `All`
+    /// - If either is `None`, the result is the other
+    /// - If both are `Packages`, the result is the union of the two
+    pub fn union(&self, other: &NoBinary) -> NoBinary {
+        match (self, other) {
+            (NoBinary::All, _) | (_, NoBinary::All) => NoBinary::All,
+            (NoBinary::None, _) => other.clone(),
+            (_, NoBinary::None) => self.clone(),
+            (NoBinary::Packages(packages), NoBinary::Packages(other_packages)) => {
+                let mut packages = packages.clone();
+                packages.extend(other_packages.iter().cloned());
+                NoBinary::Packages(packages)
+            }
+        }
+    }
+}
+
 /// Specific options for a PyPI registries
 #[derive(Debug, Clone, PartialEq, Serialize, Eq, Default)]
 #[serde(rename_all = "kebab-case")]
@@ -101,6 +135,10 @@ pub struct PypiOptions {
     pub index_strategy: Option<IndexStrategy>,
     /// Don't build sdist for all or certain packages
     pub no_build: Option<NoBuild>,
+    /// Dependency overrides
+    pub dependency_overrides: Option<IndexMap<PypiPackageName, PixiPypiSpec>>,
+    /// Don't use pre-built wheels all or certain packages
+    pub no_binary: Option<NoBinary>,
 }
 
 /// Clones and deduplicates two iterators of values
@@ -117,6 +155,7 @@ fn clone_and_deduplicate<'a, I: Iterator<Item = &'a T>, T: Clone + Eq + Hash + '
 }
 
 impl PypiOptions {
+    #[allow(clippy::too_many_arguments)]
     pub fn new(
         index: Option<Url>,
         extra_indexes: Option<Vec<Url>>,
@@ -124,6 +163,8 @@ impl PypiOptions {
         no_build_isolation: NoBuildIsolation,
         index_strategy: Option<IndexStrategy>,
         no_build: Option<NoBuild>,
+        dependency_overrides: Option<IndexMap<PypiPackageName, PixiPypiSpec>>,
+        no_binary: Option<NoBinary>,
     ) -> Self {
         Self {
             index_url: index,
@@ -132,6 +173,8 @@ impl PypiOptions {
             no_build_isolation,
             index_strategy,
             no_build,
+            dependency_overrides,
+            no_binary,
         }
     }
 
@@ -235,6 +278,29 @@ impl PypiOptions {
             (None, Some(b)) => Some(b.clone()),
             (None, None) => None,
         };
+        // Set the dependency overrides
+        // notice that default feature comes last in the feature_ext
+        // so we want self overwriting the other
+        // i.e. if same key exists in both, we want the value from `self` to be used
+        // so we extend b with a (a overrides b)
+        let dependency_overrides = match (&self.dependency_overrides, &other.dependency_overrides) {
+            (Some(a), Some(b)) => {
+                let mut overrides = b.clone();
+                overrides.extend(a.into_iter().map(|(k, v)| (k.clone(), v.clone())));
+                Some(overrides)
+            }
+            (Some(a), None) => Some(a.clone()),
+            (None, Some(b)) => Some(b.clone()),
+            (None, None) => None,
+        };
+
+        // Set the no-binary option
+        let no_binary = match (self.no_binary.as_ref(), other.no_binary.as_ref()) {
+            (Some(a), Some(b)) => Some(a.union(b)),
+            (Some(a), None) => Some(a.clone()),
+            (None, Some(b)) => Some(b.clone()),
+            (None, None) => None,
+        };
 
         Ok(PypiOptions {
             index_url: index,
@@ -243,6 +309,8 @@ impl PypiOptions {
             no_build_isolation,
             index_strategy,
             no_build,
+            dependency_overrides,
+            no_binary,
         })
     }
 }
@@ -396,6 +464,17 @@ mod tests {
             ]),
             index_strategy: None,
             no_build: None,
+            dependency_overrides:Some(IndexMap::from_iter([
+                (
+                    "pkg1".parse().unwrap(),
+                    PixiPypiSpec::RawVersion("==1.0.0".parse().unwrap()),
+                ),
+                (
+                    "pkg2".parse().unwrap(),
+                    PixiPypiSpec::RawVersion("==2.0.0".parse().unwrap()),
+                ),
+            ])),
+            no_binary: Default::default(),
         };
 
         // Create the second set of options
@@ -409,6 +488,18 @@ mod tests {
             no_build_isolation: NoBuildIsolation::from_iter(["foo".parse().unwrap()]),
             index_strategy: None,
             no_build: Some(NoBuild::All),
+            dependency_overrides: Some(IndexMap::from_iter([
+                (
+                    "pkg1".parse().unwrap(),
+                    PixiPypiSpec::RawVersion("==1.2.0".parse().unwrap()),
+                ),
+                (
+                    "pkg3".parse().unwrap(),
+                    PixiPypiSpec::RawVersion("==3.2.0".parse().unwrap()),
+                ),
+            ])),
+
+            no_binary: Default::default(),
         };
 
         // Merge the two options
@@ -427,27 +518,65 @@ mod tests {
         assert_eq!(NoBuild::All.union(&NoBuild::None), NoBuild::All);
         assert_eq!(NoBuild::None.union(&NoBuild::All), NoBuild::All);
         assert_eq!(
-            NoBuild::All.union(&NoBuild::Packages(HashSet::from_iter([pkg1.clone()]))),
+            NoBuild::All.union(&NoBuild::Packages(IndexSet::from_iter([pkg1.clone()]))),
             NoBuild::All
         );
 
         // Case 2: One is `None`, result should be the other
         assert_eq!(NoBuild::None.union(&NoBuild::None), NoBuild::None);
         assert_eq!(
-            NoBuild::None.union(&NoBuild::Packages(HashSet::from_iter([pkg1.clone()]))),
-            NoBuild::Packages(HashSet::from_iter([pkg1.clone()]))
+            NoBuild::None.union(&NoBuild::Packages(IndexSet::from_iter([pkg1.clone()]))),
+            NoBuild::Packages(IndexSet::from_iter([pkg1.clone()]))
         );
         assert_eq!(
-            NoBuild::Packages(HashSet::from_iter([pkg1.clone()])).union(&NoBuild::None),
-            NoBuild::Packages(HashSet::from_iter([pkg1.clone()]))
+            NoBuild::Packages(IndexSet::from_iter([pkg1.clone()])).union(&NoBuild::None),
+            NoBuild::Packages(IndexSet::from_iter([pkg1.clone()]))
         );
 
         // Case 3: Both are `Packages`, result should be the union of the two
         assert_eq!(
-            NoBuild::Packages(HashSet::from_iter([pkg1.clone(), pkg2.clone()])).union(
-                &NoBuild::Packages(HashSet::from_iter([pkg2.clone(), pkg3.clone()]))
+            NoBuild::Packages(IndexSet::from_iter([pkg1.clone(), pkg2.clone()])).union(
+                &NoBuild::Packages(IndexSet::from_iter([pkg2.clone(), pkg3.clone()]))
             ),
-            NoBuild::Packages(HashSet::from_iter([
+            NoBuild::Packages(IndexSet::from_iter([
+                pkg1.clone(),
+                pkg2.clone(),
+                pkg3.clone()
+            ]))
+        );
+    }
+
+    #[test]
+    fn test_no_binary_union() {
+        let pkg1 = pep508_rs::PackageName::new("pkg1".to_string()).unwrap();
+        let pkg2 = pep508_rs::PackageName::new("pkg1".to_string()).unwrap();
+        let pkg3 = pep508_rs::PackageName::new("pkg1".to_string()).unwrap();
+
+        // Case 1: One is `All`, result should be `All`
+        assert_eq!(NoBinary::All.union(&NoBinary::None), NoBinary::All);
+        assert_eq!(NoBinary::None.union(&NoBinary::All), NoBinary::All);
+        assert_eq!(
+            NoBinary::All.union(&NoBinary::Packages(IndexSet::from_iter([pkg1.clone()]))),
+            NoBinary::All
+        );
+
+        // Case 2: One is `None`, result should be the other
+        assert_eq!(NoBinary::None.union(&NoBinary::None), NoBinary::None);
+        assert_eq!(
+            NoBinary::None.union(&NoBinary::Packages(IndexSet::from_iter([pkg1.clone()]))),
+            NoBinary::Packages(IndexSet::from_iter([pkg1.clone()]))
+        );
+        assert_eq!(
+            NoBinary::Packages(IndexSet::from_iter([pkg1.clone()])).union(&NoBinary::None),
+            NoBinary::Packages(IndexSet::from_iter([pkg1.clone()]))
+        );
+
+        // Case 3: Both are `Packages`, result should be the union of the two
+        assert_eq!(
+            NoBinary::Packages(IndexSet::from_iter([pkg1.clone(), pkg2.clone()])).union(
+                &NoBinary::Packages(IndexSet::from_iter([pkg2.clone(), pkg3.clone()]))
+            ),
+            NoBinary::Packages(IndexSet::from_iter([
                 pkg1.clone(),
                 pkg2.clone(),
                 pkg3.clone()
@@ -465,6 +594,8 @@ mod tests {
             no_build_isolation: NoBuildIsolation::default(),
             index_strategy: None,
             no_build: Default::default(),
+            dependency_overrides: None,
+            no_binary: Default::default(),
         };
 
         // Create the second set of options
@@ -475,6 +606,8 @@ mod tests {
             no_build_isolation: NoBuildIsolation::default(),
             index_strategy: None,
             no_build: Default::default(),
+            dependency_overrides: None,
+            no_binary: Default::default(),
         };
 
         // Merge the two options
@@ -493,6 +626,8 @@ mod tests {
             no_build_isolation: NoBuildIsolation::default(),
             index_strategy: Some(IndexStrategy::FirstIndex),
             no_build: Default::default(),
+            dependency_overrides: None,
+            no_binary: Default::default(),
         };
 
         // Create the second set of options
@@ -503,6 +638,8 @@ mod tests {
             no_build_isolation: NoBuildIsolation::default(),
             index_strategy: Some(IndexStrategy::UnsafeBestMatch),
             no_build: Default::default(),
+            dependency_overrides: None,
+            no_binary: Default::default(),
         };
 
         // Merge the two options
