@@ -3,13 +3,12 @@ use std::{
     str::FromStr,
 };
 
+use crate::GitUrlWithPrefix;
 use miette::IntoDiagnostic;
 use pep440_rs::VersionSpecifiers;
-use pixi_git::{
-    git::GitReference as PixiGitReference, sha::GitSha as PixiGitSha, url::RepositoryUrl,
-};
+use pixi_git::{git::GitReference as PixiGitReference, sha::GitSha as PixiGitSha};
 use pixi_manifest::pypi::pypi_options::{
-    FindLinksUrlOrPath, IndexStrategy, NoBuild, NoBuildIsolation, PypiOptions,
+    FindLinksUrlOrPath, IndexStrategy, NoBinary, NoBuild, NoBuildIsolation, PypiOptions,
 };
 use pixi_record::{LockedGitUrl, PinnedGitCheckout, PinnedGitSpec};
 use pixi_spec::GitReference as PixiReference;
@@ -17,6 +16,7 @@ use uv_configuration::BuildOptions;
 use uv_distribution_types::{GitSourceDist, Index, IndexLocations, IndexUrl};
 use uv_pep508::{InvalidNameError, PackageName, VerbatimUrl, VerbatimUrlError};
 use uv_python::PythonEnvironment;
+use uv_redacted::DisplaySafeUrl;
 
 use crate::{ConversionError, VersionError};
 
@@ -29,7 +29,10 @@ pub enum ConvertFlatIndexLocationError {
 }
 
 /// Convert PyPI options to build options
-pub fn no_build_to_build_options(no_build: &NoBuild) -> Result<BuildOptions, InvalidNameError> {
+pub fn pypi_options_to_build_options(
+    no_build: &NoBuild,
+    no_binary: &NoBinary,
+) -> Result<BuildOptions, InvalidNameError> {
     let uv_no_build = match no_build {
         NoBuild::None => uv_configuration::NoBuild::None,
         NoBuild::All => uv_configuration::NoBuild::All,
@@ -39,10 +42,17 @@ pub fn no_build_to_build_options(no_build: &NoBuild) -> Result<BuildOptions, Inv
                 .collect::<Result<Vec<_>, _>>()?,
         ),
     };
-    Ok(BuildOptions::new(
-        uv_configuration::NoBinary::default(),
-        uv_no_build,
-    ))
+    let uv_no_binary = match no_binary {
+        NoBinary::None => uv_configuration::NoBinary::None,
+        NoBinary::All => uv_configuration::NoBinary::All,
+        NoBinary::Packages(vec) => uv_configuration::NoBinary::Packages(
+            vec.iter()
+                .map(|s| PackageName::from_str(s.as_ref()))
+                .collect::<Result<Vec<_>, _>>()?,
+        ),
+    };
+
+    Ok(BuildOptions::new(uv_no_binary, uv_no_build))
 }
 
 /// Convert the subset of pypi-options to index locations
@@ -62,6 +72,7 @@ pub fn pypi_options_to_index_locations(
     let index = options
         .index_url
         .clone()
+        .map(DisplaySafeUrl::from)
         .map(VerbatimUrl::from_url)
         .map(IndexUrl::from)
         .map(Index::from_index_url)
@@ -74,6 +85,7 @@ pub fn pypi_options_to_index_locations(
         .into_iter()
         .flat_map(|urls| {
             urls.into_iter()
+                .map(DisplaySafeUrl::from)
                 .map(VerbatimUrl::from_url)
                 .map(IndexUrl::from)
                 .map(Index::from_extra_index_url)
@@ -86,7 +98,7 @@ pub fn pypi_options_to_index_locations(
             .map(|url| match url {
                 FindLinksUrlOrPath::Path(relative) => VerbatimUrl::from_path(&relative, base_path)
                     .map_err(|e| ConvertFlatIndexLocationError::VerbatimUrlError(e, relative)),
-                FindLinksUrlOrPath::Url(url) => Ok(VerbatimUrl::from_url(url.clone())),
+                FindLinksUrlOrPath::Url(url) => Ok(VerbatimUrl::from_url(url.clone().into())),
             })
             .collect::<Result<Vec<_>, _>>()?
             .into_iter()
@@ -122,6 +134,7 @@ pub fn locked_indexes_to_index_locations(
         .indexes
         .first()
         .cloned()
+        .map(DisplaySafeUrl::from)
         .map(VerbatimUrl::from_url)
         .map(IndexUrl::from)
         .map(Index::from_index_url)
@@ -131,6 +144,7 @@ pub fn locked_indexes_to_index_locations(
         .iter()
         .skip(1)
         .cloned()
+        .map(DisplaySafeUrl::from)
         .map(VerbatimUrl::from_url)
         .map(IndexUrl::from)
         .map(Index::from_extra_index_url);
@@ -143,7 +157,9 @@ pub fn locked_indexes_to_index_locations(
                     ConvertFlatIndexLocationError::VerbatimUrlError(e, relative.clone())
                 })
             }
-            rattler_lock::FindLinksUrlOrPath::Url(url) => Ok(VerbatimUrl::from_url(url.clone())),
+            rattler_lock::FindLinksUrlOrPath::Url(url) => {
+                Ok(VerbatimUrl::from_url(url.clone().into()))
+            }
         })
         .collect::<Result<Vec<_>, _>>()?
         .into_iter()
@@ -278,7 +294,7 @@ pub fn into_pinned_git_spec(dist: GitSourceDist) -> PinnedGitSpec {
         reference,
     );
 
-    PinnedGitSpec::new(dist.git.repository().clone(), pinned_checkout)
+    PinnedGitSpec::new(dist.git.repository().clone().into(), pinned_checkout)
 }
 
 /// Convert a locked git url into a parsed git url
@@ -297,7 +313,16 @@ pub fn to_parsed_git_url(
     // Construct manually [`ParsedGitUrl`] from locked url.
     let parsed_git_url = uv_pypi_types::ParsedGitUrl::from_source(
         uv_git_types::GitUrl::from_fields(
-            RepositoryUrl::new(&locked_git_url.to_url()).into(),
+            {
+                let mut url = locked_git_url.to_url();
+                // Locked git url contains query parameters and fragments
+                // so we need to clean it to a base repository URL
+                url.set_fragment(None);
+                url.set_query(None);
+
+                let git_url = GitUrlWithPrefix::from(&url);
+                git_url.to_display_safe_url()
+            },
             into_uv_git_reference(git_source.reference.into()),
             Some(into_uv_git_sha(git_source.commit)),
         )
