@@ -487,3 +487,88 @@ def test_help_warning_when_platform_not_supported(pixi: Path, tmp_pixi_workspace
         ExitCode.COMMAND_NOT_FOUND,
         stderr_contains=["pixi workspace platform add"],
     )
+
+
+@pytest.mark.slow
+def test_issue_4123_cache_prevents_editable_install(
+    pixi: Path, tmp_pixi_workspace: Path, tmp_path: Path
+) -> None:
+    """
+    Reproduce issue #4123: cached registry package prevents proper editable installation
+    of same name/version from local source.
+
+    In this issue when installing a wheel from PyPI, that would be cached, when subsequently (even in a different project)
+    using a local version of the same package name and version, it would use the cached registry version instead of the local source.
+    This is basically a reproduction of the linked issue.
+    """
+    # Use shared cache directory for all pixi commands,
+    # so that we have no cache interference
+    cache_env = {"PIXI_CACHE_DIR": str(tmp_path / "pixi_cache")}
+
+    # Create local package with same name/version as will be installed from PyPI
+    local_pkg = tmp_pixi_workspace / "local_project"
+    local_pkg.mkdir(parents=True)
+
+    (local_pkg / "pyproject.toml").write_text("""[project]
+name = "six"
+version = "1.16.0"
+[build-system]
+requires = ["hatchling"]
+build-backend = "hatchling.build"
+""")
+
+    src_dir = local_pkg / "six"
+    src_dir.mkdir()
+    # Write a simple module with a local marker
+    (src_dir / "__init__.py").write_text('__version__ = "1.16.0"\nlocal_marker = "LOCAL"')
+
+    # Create a new pixi workspace
+    # Step 1: Install from PyPI (caches the package)
+    proj1 = tmp_pixi_workspace / "proj1"
+    proj1.mkdir()
+
+    (proj1 / "pixi.toml").write_text(f"""[project]
+name = "proj1"
+channels = ["conda-forge"]
+platforms = ["{CURRENT_PLATFORM}"]
+[dependencies]
+python = "3.12.*"
+[pypi-dependencies]
+six = "==1.16.0"
+""")
+    verify_cli_command(
+        [pixi, "install", "--manifest-path", proj1 / "pixi.toml"], ExitCode.SUCCESS, env=cache_env
+    )
+
+    # Step 2: Install same name/version as editable from local source
+    proj2 = tmp_pixi_workspace / "proj2"
+    proj2.mkdir()
+
+    (proj2 / "pixi.toml").write_text(f"""[project]
+name = "proj2"
+channels = ["conda-forge"]
+platforms = ["{CURRENT_PLATFORM}"]
+[dependencies]
+python = "3.12.*"
+[pypi-dependencies]
+six = {{ path = "../local_project", editable = true }}
+""")
+
+    verify_cli_command(
+        [pixi, "install", "--manifest-path", proj2 / "pixi.toml"], ExitCode.SUCCESS, env=cache_env
+    )
+    # Step 3: Verify we get local version (has local_marker), not cached registry version
+    verify_cli_command(
+        [
+            pixi,
+            "run",
+            "--manifest-path",
+            proj2 / "pixi.toml",
+            "python",
+            "-c",
+            "import six; print(six.local_marker)",
+        ],
+        ExitCode.SUCCESS,
+        stdout_contains="LOCAL",
+        env=cache_env,
+    )
