@@ -500,7 +500,7 @@ impl PixiControl {
         // Ensure the lock-file is up-to-date
         let lock_file = project
             .update_lock_file(UpdateLockFileOptions {
-                lock_file_usage: args.lock_file_update_config.lock_file_usage(),
+                lock_file_usage: args.lock_file_update_config.lock_file_usage()?,
                 ..UpdateLockFileOptions::default()
             })
             .await?;
